@@ -1,8 +1,7 @@
 /* gdbm.h  -  The include file for dbm users.  -*- c -*- */
 
 /*  This file is part of GDBM, the GNU data base manager, by Philip A. Nelson.
-    Copyright (C) 1990-1991, 1993, 2011, 2016-2018 Free Software
-    Foundation, Inc.
+    Copyright (C) 1990-2022 Free Software Foundation, Inc.
 
     GDBM is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +30,7 @@
 # define _GDBM_H_
 
 # include <stdio.h>
+# include <sys/types.h>
 
 /* GDBM C++ support */
 # if defined(__cplusplus) || defined(c_plusplus)
@@ -45,24 +45,28 @@ extern "C" {
 # define GDBM_NEWDB	3	/* A writer.  Always create a new db. */
 # define GDBM_OPENMASK	7	/* Mask for the above. */
 
-# define GDBM_FAST	0x010	/* Write fast! => No fsyncs.  OBSOLETE. */
-# define GDBM_SYNC	0x020	/* Sync operations to the disk. */
-# define GDBM_NOLOCK	0x040	/* Don't do file locking operations. */
-# define GDBM_NOMMAP	0x080	/* Don't use mmap(). */
-# define GDBM_CLOEXEC   0x100   /* Close the underlying fd on exec(3) */
-# define GDBM_BSEXACT   0x200   /* Don't adjust block_size. Bail out with
+# define GDBM_FAST	0x0010	/* Write fast! => No fsyncs.  OBSOLETE. */
+# define GDBM_SYNC	0x0020	/* Sync operations to the disk. */
+# define GDBM_NOLOCK	0x0040	/* Don't do file locking operations. */
+# define GDBM_NOMMAP	0x0080	/* Don't use mmap(). */
+# define GDBM_CLOEXEC   0x0100  /* Close the underlying fd on exec(3) */
+# define GDBM_BSEXACT   0x0200  /* Don't adjust block_size. Bail out with
 				   GDBM_BLOCK_SIZE_ERROR error if unable to
 				   set it. */  
-# define GDBM_CLOERROR  0x400   /* Only for gdbm_fd_open: close fd on error. */
+# define GDBM_CLOERROR  0x0400  /* Only for gdbm_fd_open: close fd on error. */
+# define GDBM_XVERIFY   0x0800  /* Additional consistency checks. */
+# define GDBM_PREREAD   0x1000  /* Enable pre-fault reading of mmapped regions. */
+# define GDBM_NUMSYNC   0x2000  /* Enable the numsync extension */
+
   
 /* Parameters to gdbm_store for simple insertion or replacement in the
    case that the key is already in the database. */
 # define GDBM_INSERT	0	/* Never replace old data with new. */
 # define GDBM_REPLACE	1	/* Always replace old data with new. */
 
-/* Parameters to gdbm_setopt, specifing the type of operation to perform. */
+/* Parameters to gdbm_setopt, specifying the type of operation to perform. */
 # define GDBM_SETCACHESIZE    1  /* Set the cache size. */
-# define GDBM_FASTMODE	      2	/* Toggle fast mode.  OBSOLETE. */
+# define GDBM_FASTMODE	      2	 /* Toggle fast mode.  OBSOLETE. */
 # define GDBM_SETSYNCMODE     3  /* Turn on or off sync operations. */
 # define GDBM_SETCENTFREE     4  /* Keep all free blocks in the header. */
 # define GDBM_SETCOALESCEBLKS 5  /* Attempt to coalesce free blocks. */
@@ -84,9 +88,18 @@ extern "C" {
 # define GDBM_GETMAXMAPSIZE   14 /* Get maximum mapped memory size */
 # define GDBM_GETDBNAME       15 /* Return database file name */
 # define GDBM_GETBLOCKSIZE    16 /* Return block size */
+# define GDBM_GETDBFORMAT     17 /* Return the database format */
+# define GDBM_GETDIRDEPTH     18 /* Directory depth: number of initial (most
+				    significant) bits in hash interpreted as
+				    index to the directory. */
+# define GDBM_GETBUCKETSIZE   19 /* Get number of elements per bucket */
+# define GDBM_GETCACHEAUTO    20 /* Get the value of cache auto-adjustment */
+# define GDBM_SETCACHEAUTO    21 /* Set the value of cache auto-adjustment */
+    
+# define GDBM_CACHE_AUTO      0
 
 typedef unsigned long long int gdbm_count_t;
-  
+
 /* The data and key structure. */
 typedef struct
 {
@@ -94,7 +107,6 @@ typedef struct
   int   dsize;
 } datum;
 
-
 /* A pointer to the GDBM file. */
 typedef struct gdbm_file_info *GDBM_FILE;
 
@@ -102,7 +114,7 @@ typedef struct gdbm_file_info *GDBM_FILE;
 extern const char *gdbm_version;	
 
 # define GDBM_VERSION_MAJOR 1
-# define GDBM_VERSION_MINOR 16
+# define GDBM_VERSION_MINOR 23
 # define GDBM_VERSION_PATCH 0
 
 extern int const gdbm_version_number[3];
@@ -113,7 +125,7 @@ extern GDBM_FILE gdbm_fd_open (int fd, const char *file_name, int block_size,
 			       int flags, void (*fatal_func) (const char *));
 extern GDBM_FILE gdbm_open (const char *, int, int, int,
 			    void (*)(const char *));
-extern void gdbm_close (GDBM_FILE);
+extern int gdbm_close (GDBM_FILE);
 extern int gdbm_store (GDBM_FILE, datum, datum, int);
 extern datum gdbm_fetch (GDBM_FILE, datum);
 extern int gdbm_delete (GDBM_FILE, datum);
@@ -121,7 +133,21 @@ extern datum gdbm_firstkey (GDBM_FILE);
 extern datum gdbm_nextkey (GDBM_FILE, datum);
 extern int gdbm_reorganize (GDBM_FILE);
   
-extern void gdbm_sync (GDBM_FILE);
+extern int gdbm_sync (GDBM_FILE);
+extern int gdbm_failure_atomic (GDBM_FILE, const char *, const char *);
+
+extern int gdbm_convert (GDBM_FILE dbf, int flag);
+  
+enum gdbm_latest_snapshot_status
+  {
+    GDBM_SNAPSHOT_OK,        /* Selected the right snapshot. */
+    GDBM_SNAPSHOT_BAD,       /* Neither snapshot is readable. */
+    GDBM_SNAPSHOT_ERR,       /* Error selecting snapshot. Inspect errno. */
+    GDBM_SNAPSHOT_SAME,      /* Snapshot numsync and dates are the same. */
+    GDBM_SNAPSHOT_SUSPICIOUS /* Selected snapshot is unreliable: numsyncs
+				differ by more than 1. */
+  };
+extern int gdbm_latest_snapshot (const char *, const char *, const char **);
 extern int gdbm_exists (GDBM_FILE, datum);
 extern int gdbm_setopt (GDBM_FILE, int, void *, int);
 extern int gdbm_fdesc (GDBM_FILE);
@@ -133,6 +159,9 @@ extern int gdbm_import (GDBM_FILE, const char *, int);
 extern int gdbm_import_from_file (GDBM_FILE dbf, FILE *fp, int flag);
 
 extern int gdbm_count (GDBM_FILE dbf, gdbm_count_t *pcount);
+extern int gdbm_bucket_count (GDBM_FILE dbf, size_t *pcount);
+
+extern int gdbm_avail_verify (GDBM_FILE dbf);
 
 typedef struct gdbm_recovery_s
 {
@@ -188,46 +217,59 @@ extern int gdbm_load_from_file (GDBM_FILE *, FILE *, int replace,
 
 extern int gdbm_copy_meta (GDBM_FILE dst, GDBM_FILE src);
 
-# define GDBM_NO_ERROR		         0
-# define GDBM_MALLOC_ERROR	         1
-# define GDBM_BLOCK_SIZE_ERROR	         2
-# define GDBM_FILE_OPEN_ERROR	         3
-# define GDBM_FILE_WRITE_ERROR	         4
-# define GDBM_FILE_SEEK_ERROR	         5
-# define GDBM_FILE_READ_ERROR	         6
-# define GDBM_BAD_MAGIC_NUMBER	         7
-# define GDBM_EMPTY_DATABASE	         8
-# define GDBM_CANT_BE_READER	         9
-# define GDBM_CANT_BE_WRITER	        10
-# define GDBM_READER_CANT_DELETE	11
-# define GDBM_READER_CANT_STORE	        12
-# define GDBM_READER_CANT_REORGANIZE	13
-# define GDBM_UNKNOWN_ERROR	        14
-# define GDBM_ITEM_NOT_FOUND	        15
-# define GDBM_REORGANIZE_FAILED	        16
-# define GDBM_CANNOT_REPLACE	        17
-# define GDBM_ILLEGAL_DATA	        18
-# define GDBM_OPT_ALREADY_SET	        19
-# define GDBM_OPT_ILLEGAL           	20
-# define GDBM_BYTE_SWAPPED	        21
-# define GDBM_BAD_FILE_OFFSET	        22
-# define GDBM_BAD_OPEN_FLAGS	        23
-# define GDBM_FILE_STAT_ERROR           24
-# define GDBM_FILE_EOF                  25
-# define GDBM_NO_DBNAME                 26
-# define GDBM_ERR_FILE_OWNER            27
-# define GDBM_ERR_FILE_MODE             28
-# define GDBM_NEED_RECOVERY             29
-# define GDBM_BACKUP_FAILED             30
-# define GDBM_DIR_OVERFLOW              31
-# define GDBM_BAD_BUCKET                32
-# define GDBM_BAD_HEADER                33
-# define GDBM_BAD_AVAIL                 34
-# define GDBM_BAD_HASH_TABLE            35
-# define GDBM_BAD_DIR_ENTRY             36
-
+enum
+  {
+    GDBM_NO_ERROR		 = 0,
+    GDBM_MALLOC_ERROR	         = 1,
+    GDBM_BLOCK_SIZE_ERROR	 = 2,
+    GDBM_FILE_OPEN_ERROR	 = 3,
+    GDBM_FILE_WRITE_ERROR	 = 4,
+    GDBM_FILE_SEEK_ERROR	 = 5,
+    GDBM_FILE_READ_ERROR	 = 6,
+    GDBM_BAD_MAGIC_NUMBER	 = 7,
+    GDBM_EMPTY_DATABASE	         = 8,
+    GDBM_CANT_BE_READER	         = 9,
+    GDBM_CANT_BE_WRITER	         = 10, 
+    GDBM_READER_CANT_DELETE	 = 11,
+    GDBM_READER_CANT_STORE	 = 12,
+    GDBM_READER_CANT_REORGANIZE	 = 13,
+    GDBM_UNKNOWN_ERROR	         = 14,
+    GDBM_ITEM_NOT_FOUND	         = 15,
+    GDBM_REORGANIZE_FAILED	 = 16,
+    GDBM_CANNOT_REPLACE	         = 17,
+    GDBM_MALFORMED_DATA	         = 18,
+    GDBM_ILLEGAL_DATA            = GDBM_MALFORMED_DATA,
+    GDBM_OPT_ALREADY_SET	 = 19,
+    GDBM_OPT_BADVAL           	 = 20,
+    GDBM_OPT_ILLEGAL           	 = GDBM_OPT_BADVAL,
+    GDBM_BYTE_SWAPPED	         = 21,
+    GDBM_BAD_FILE_OFFSET	 = 22,
+    GDBM_BAD_OPEN_FLAGS	         = 23,
+    GDBM_FILE_STAT_ERROR         = 24,
+    GDBM_FILE_EOF                = 25,
+    GDBM_NO_DBNAME               = 26,
+    GDBM_ERR_FILE_OWNER          = 27,
+    GDBM_ERR_FILE_MODE           = 28,
+    GDBM_NEED_RECOVERY           = 29,
+    GDBM_BACKUP_FAILED           = 30,
+    GDBM_DIR_OVERFLOW            = 31,
+    GDBM_BAD_BUCKET              = 32,
+    GDBM_BAD_HEADER              = 33,
+    GDBM_BAD_AVAIL               = 34,
+    GDBM_BAD_HASH_TABLE          = 35,
+    GDBM_BAD_DIR_ENTRY           = 36,
+    GDBM_FILE_CLOSE_ERROR        = 37, 
+    GDBM_FILE_SYNC_ERROR         = 38,
+    GDBM_FILE_TRUNCATE_ERROR     = 39,
+    GDBM_BUCKET_CACHE_CORRUPTED  = 40,
+    GDBM_BAD_HASH_ENTRY          = 41,
+    GDBM_ERR_SNAPSHOT_CLONE      = 42,
+    GDBM_ERR_REALPATH            = 43,
+    GDBM_ERR_USAGE               = 44
+  };
+  
 # define _GDBM_MIN_ERRNO	0
-# define _GDBM_MAX_ERRNO	GDBM_BAD_DIR_ENTRY
+# define _GDBM_MAX_ERRNO	GDBM_ERR_USAGE
 
 /* This one was never used and will be removed in the future */
 # define GDBM_UNKNOWN_UPDATE GDBM_UNKNOWN_ERROR
@@ -272,8 +314,21 @@ extern void gdbm_debug_parse_state (int (*f) (void *, int, char const *),
 				    void *d);
   
 extern void gdbm_debug_datum (datum dat, char const *pfx);
-  
 #endif
+
+/* Cache statistics */  
+struct gdbm_cache_stat
+{
+  off_t adr;
+  size_t hits;
+};
+
+void gdbm_get_cache_stats (GDBM_FILE dbf,
+			   size_t *access_count,
+			   size_t *cache_hits,
+			   size_t *cache_count,
+			   struct gdbm_cache_stat *bstat,
+			   size_t nstat);
   
 # if defined(__cplusplus) || defined(c_plusplus)
 }
